#include "WProgram.h"
void shiftOut();
void shiftByte(int myDataPin, int myClockPin, byte myDataOut, byte myByteCopys);
void blinkAll(int n, int d);
/*
| Digigripper
| Olaf Val
| www.olafval.de
|
| based on shiftOutCode, Predefined Array Style
| by Carlyn Maw, Tom Igoe 
*/


//Pin connected to ST_CP of 74HC595
int latchPin = 8;
//Pin connected to SH_CP of 74HC595
int clockPin = 12;
////Pin connected to DS of 74HC595
int dataPin = 11;

//holders for infromation you're going to pass to shifting function
byte data;
byte dataArray[2];

byte incomingByte;


void setup() {
  
  Serial.begin(9600);	// opens serial port, sets data rate to 9600 bps
  
  //set pins to output because they are addressed in the main loop
  pinMode(latchPin, OUTPUT);

  //function that blinks all the LEDs
  //gets passed the number of blinks and the pause time
  blinkAll(2,500); 
}

void loop() {
  
  // send data only when you receive data:
  if (Serial.available() > 0) {
    // read the incoming byte:
    incomingByte = Serial.read();
    
    if (incomingByte == 65) {
      //A
      dataArray[0] = 0xF0; //11110000
      dataArray[1] = 0x0; //00000000  
    }
    
    if (incomingByte == 66) {
      //B
      dataArray[0] = 0xF; //00001111
      dataArray[1] = 0x0; //00000000  
    }
    
    if (incomingByte == 67) {
      //C
      dataArray[0] = 0; //00000000
      dataArray[1] = 240; //11110000  
    }
    
    if (incomingByte == 68) {
      //D
      dataArray[0] = 0; //00000000
      dataArray[1] = 15; //00001111  
    }
   
    //Arduino doesn't seem to have a way to write binary straight into the code 
    //so these values are in Decimal. 
    
    shiftOut();
  }
  
}

void shiftOut(){
  for (int j = 0; j < 2; j++) {
    //load the light sequence you want from array
    data = dataArray[j];
    //ground latchPin and hold low for as long as you are transmitting
    digitalWrite(latchPin, 0);
    //move 'em out
    shiftByte(dataPin, clockPin, data, 1);   
    //return the latch pin high to signal chip that it 
    //no longer needs to listen for information
    digitalWrite(latchPin, 1);
  }
}

void shiftByte(int myDataPin, int myClockPin, byte myDataOut, byte myByteCopys) {
  // This shifts 8 bits out MSB first, 
  //on the rising edge of the clock,
  //clock idles low

  //internal function setup
  int b;
  int i=0;
  int pinState;
  pinMode(myClockPin, OUTPUT);
  pinMode(myDataPin, OUTPUT);

  //clear everything out just in case to
  //prepare shift register for bit shifting
  digitalWrite(myDataPin, 0);
  digitalWrite(myClockPin, 0);

  //for each bit in the byte myDataOut\u2026
  //NOTICE THAT WE ARE COUNTING DOWN in our for loop
  //This means that %00000001 or "1" will go through such
  //that it will be pin Q0 that lights. 
  for (b=1; b<=myByteCopys; b++){
    
    for (i=7; i>=0; i--)  {
      digitalWrite(myClockPin, 0);

      //if the value passed to myDataOut and a bitmask result 
      // true then... so if we are at i=6 and our value is
      // %11010100 it would the code compares it to %01000000 
      // and proceeds to set pinState to 1.
      if ( myDataOut & (1<<i) ) {
        pinState= 1;
      }
      else {	
        pinState= 0;
      }

      //Sets the pin to HIGH or LOW depending on pinState
      digitalWrite(myDataPin, pinState);
      //register shifts bits on upstroke of clock pin  
      digitalWrite(myClockPin, 1);
      //zero the data pin after shift to prevent bleed through
      digitalWrite(myDataPin, 0);
    }
  }
  //stop shifting
  digitalWrite(myClockPin, 0);
}


//blinks the whole register based on the number of times you want to 
//blink "n" and the pause between them "d"
//starts with a moment of darkness to make sure the first blink
//has its full visual effect.
void blinkAll(int n, int d) {
  digitalWrite(latchPin, 0);
  shiftOut(dataPin, clockPin, 0, 1);
  digitalWrite(latchPin, 1);
  delay(200);
  for (int x = 0; x < n; x++) {
    digitalWrite(latchPin, 0);
    shiftOut(dataPin, clockPin, 255, 1);
    digitalWrite(latchPin, 1);
    delay(d);
    digitalWrite(latchPin, 0);
    shiftOut(dataPin, clockPin, 0, 1);
    digitalWrite(latchPin, 1);
    delay(d);
  }
}
int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

